/*****************************************************************************
 *                   SEGGER Microcontroller GmbH & Co. KG                    *
 *            Solutions for real time microcontroller applications           *
 *****************************************************************************
 *                                                                           *
 *               (c) 2015 SEGGER Microcontroller GmbH & Co. KG               *
 *                                                                           *
 *           Internet: www.segger.com   Support: support@segger.com          *
 *                                                                           *
 *****************************************************************************/

function InitClocksAndMemories() {
  var i;
  // Set up PLL A
  TargetInterface.pokeWord(0xFFFFFC28, 0x20603F08);  // MULA=97, DIVA=8 - 16.36766 Mhz/8*97 = 198.4578775
  TargetInterface.delay(10);                         // while (!(TargetInterface.peekWord(0xFFFFFC68) & 2));
  // Use PLL A as processor clock and master clock divided by 2
  TargetInterface.pokeWord(0xFFFFFC30, 0x00000102);
  TargetInterface.delay(10);                         // while (!(TargetInterface.peekWord(0xFFFFFC68) & 8));

    /// Increase JTAG frequency to 15 MHz

  // Select 32 bit data bus
  TargetInterface.pokeWord(0xFFFFF804, 0xFFFF0000);
  TargetInterface.pokeWord(0xFFFFF870, 0xFFFF0000);

  // assign EBI0_CSA1 to SDRAM
  TargetInterface.pokeWord(0xFFFFED20, TargetInterface.peekWord(0xFFFFED20) | 2);

  // SDRAM timings and configuration
  TargetInterface.pokeWord(0xFFFFE208, 0x85227259);
  TargetInterface.delay(10);

  // "All Banks Precharge"
  TargetInterface.pokeWord(0xFFFFE200, 2);
  TargetInterface.pokeWord(0x20000000, 0);
  TargetInterface.delay(10);

  // 8 "Auto Refresh Commands"
  for (i = 0; i < 8 ; i++) {
    TargetInterface.pokeWord(0xFFFFE200, 4);
    TargetInterface.pokeWord(0x20000000, 0);
  }
  // "Load Mode Register"
  TargetInterface.pokeWord(0xFFFFE200, 3);
  TargetInterface.pokeWord(0x20000000, 0);
  // Set Refresh Rate to 7.8 us
  TargetInterface.pokeWord(0xFFFFE204, 780);
  // Switch to normal mode
  TargetInterface.pokeWord(0xFFFFE200, 0);
  TargetInterface.pokeWord(0x20000000, 0);
}

function Reset() {
  TargetInterface.resetAndStop(100);
  // Make sure caches and MMU are turned off
    /// MRC p15, 0, r0, c1, c0, 0 ; read control register
    /// r0 &= ~(1<<0|1<<2|1<<12);
    /// MCR p15, 0, r0, c1, c0, 0 ; write control register
  //Enable the main oscillator
  TargetInterface.pokeWord(0xFFFFFC20, 0x00000801);  // CKGR_MOR = 801
  TargetInterface.delay(10);                         // while (!(TargetInterface.peekWord(0xFFFFFC68) & 1));
  TargetInterface.pokeWord(0xFFFFFC30, 1);           // PMC_MCKR.CSS = 1 clock source selection = Main Clock
    /// Set JTAG frequency to 5 kHz
  // Initialize clocks and memories
  InitClocksAndMemories();
  // Map external SDRAM to address zero
  rv = TargetInterface.peekWord(0x00000000);
  // MM: Shouldn't we write to SDRAM and then check if
  //     the written value can be read from address zero?
  TargetInterface.pokeWord(0x00000000, 0x87654321);
  if (TargetInterface.peekWord(0x00000000) != 0x87654321) {
    TargetInterface.pokeWord(0xFFFFED00, 0x3);
  } else {
    TargetInterface.pokeWord(0x00000000, rv);
  }
}
