/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
File    : BSP.c
Purpose : BSP for Renesas RSK RZA2M evalboard
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/
#define PORT_BASE_ADDR     (0xFCFFE000u)
#define PORT_PDR(x)        (*(unsigned short*)(PORT_BASE_ADDR + x * 2u))
#define PORT_PODR(x)       (*(unsigned char* )(PORT_BASE_ADDR + 0x40u + x))
#define PORT_PIDR(x)       (*(unsigned char* )(PORT_BASE_ADDR + 0x60u + x))
#define PORT_PMR(x)        (*(unsigned char* )(PORT_BASE_ADDR + 0x80u + x))
#define PORT_DSCR(x)       (*(unsigned short*)(PORT_BASE_ADDR + 0140u + x))

#define LED0_OFF           (1)
#define LED1_OFF           (0)

#define LED0_PORT          (0x0Cu)
#define LED1_PORT          (0x06u)

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  PORT_PDR(LED0_PORT)   |=  (3 << LED0_OFF);  // Setting LED pin to output
  PORT_PODR(LED0_PORT)  &= ~(1 << LED0_OFF);  // LED off
  PORT_PDR(LED1_PORT)   |=  (3 << LED1_OFF);  // Setting LED pin to output
  PORT_PODR(LED1_PORT)  &= ~(1 << LED1_OFF);  // LED off
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    PORT_PODR(LED0_PORT)  |= (1 << LED0_OFF);  // LED on
  } else if (Index == 1) {
    PORT_PODR(LED1_PORT)  |= (1 << LED1_OFF);  // LED on
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    PORT_PODR(LED0_PORT)  &= ~(1 << LED0_OFF);  // LED off
  } else if (Index == 1) {
    PORT_PODR(LED1_PORT)  &= ~(1 << LED1_OFF);  // LED off
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if (PORT_PODR(LED0_PORT) & (1 << LED0_OFF)) {
      PORT_PODR(LED0_PORT)  &= ~(1 << LED0_OFF);  // LED off
    } else {
      PORT_PODR(LED0_PORT)  |= (1 << LED0_OFF);  // LED on
    }
  } else if (Index == 1) {
    if (PORT_PODR(LED1_PORT) & (1 << LED1_OFF)) {
      PORT_PODR(LED1_PORT)  &= ~(1 << LED1_OFF);  // LED off
    } else {
      PORT_PODR(LED1_PORT)  |= (1 << LED1_OFF);  // LED on
    }
  }
}

/****** End Of File *************************************************/
