/*********************************************************************
*                     SEGGER Microcontroller GmbH                    *
*                        The Embedded Experts                        *
**********************************************************************
*                                                                    *
*       (c) 1995 - 2022 SEGGER Microcontroller GmbH                  *
*                                                                    *
*       Internet: segger.com  Support: support_embos@segger.com      *
*                                                                    *
**********************************************************************
*                                                                    *
*       embOS * Real time operating system                           *
*                                                                    *
*       Please note:                                                 *
*                                                                    *
*       Knowledge of this file may under no circumstances            *
*       be used to write a similar product or a real-time            *
*       operating system for in-house use.                           *
*                                                                    *
*       Thank you for your fairness !                                *
*                                                                    *
**********************************************************************
*                                                                    *
*       OS version: V5.18.0.0                                        *
*                                                                    *
**********************************************************************

-------------------------- END-OF-HEADER -----------------------------
Purpose : BSP for STM32MP157C-DK2 eval board
*/

#include "BSP.h"

/*********************************************************************
*
*       Defines
*
**********************************************************************
*/

#define LED0                (7)  // PH7  LED_Y
#define LED1               (11)  // PD11 LED_B

//
// Reset and Clock Control register (RCC)
//
#define RCC_BASE_ADDR      (0x50000000uL)
#define RCC_MP_AHB4ENSETR  (*(volatile unsigned long*)(RCC_BASE_ADDR + 0xA28uL))   // Enable for MicroProcessorUnit (MPU) set register

//
// General Purpose Input Output D (GPIOD)
//
#define GPIOD_BASE_ADDR    (0x50005000uL)
#define GPIOD_MODER        (*(volatile unsigned long*)(GPIOD_BASE_ADDR + 0x00uL))  // GPIO mode register
#define GPIOD_TYPER        (*(volatile unsigned long*)(GPIOD_BASE_ADDR + 0x04uL))  // GPIO type register
#define GPIOD_SPEEDR       (*(volatile unsigned long*)(GPIOD_BASE_ADDR + 0x08uL))  // GPIO speed register
#define GPIOD_ODR          (*(volatile unsigned long*)(GPIOD_BASE_ADDR + 0x14uL))  // GPIO output data register
#define GPIOD_BSRR         (*(volatile unsigned long*)(GPIOD_BASE_ADDR + 0x18uL))  // GPIO bit set/reset register
#define GPIOD_AFRL         (*(volatile unsigned long*)(GPIOD_BASE_ADDR + 0x20uL))  // GPIO alternate function register low
#define GPIOD_AFRH         (*(volatile unsigned long*)(GPIOD_BASE_ADDR + 0x24uL))  // GPIO alternate function register high


//
// General Purpose Input Output H (GPIOH)
//
#define GPIOH_BASE_ADDR    (0x50009000uL)
#define GPIOH_MODER        (*(volatile unsigned long*)(GPIOH_BASE_ADDR + 0x00uL))  // GPIO mode register
#define GPIOH_TYPER        (*(volatile unsigned long*)(GPIOH_BASE_ADDR + 0x04uL))  // GPIO type register
#define GPIOH_SPEEDR       (*(volatile unsigned long*)(GPIOH_BASE_ADDR + 0x08uL))  // GPIO speed register
#define GPIOH_ODR          (*(volatile unsigned long*)(GPIOH_BASE_ADDR + 0x14uL))  // GPIO output data register
#define GPIOH_BSRR         (*(volatile unsigned long*)(GPIOH_BASE_ADDR + 0x18uL))  // GPIO bit set/reset register
#define GPIOH_AFRL         (*(volatile unsigned long*)(GPIOH_BASE_ADDR + 0x20uL))  // GPIO alternate function register low
#define GPIOH_AFRH         (*(volatile unsigned long*)(GPIOH_BASE_ADDR + 0x24uL))  // GPIO alternate function register high

/*********************************************************************
*
*       Global functions
*
**********************************************************************
*/

/*********************************************************************
*
*       BSP_Init()
*/
void BSP_Init(void) {
  RCC_MP_AHB4ENSETR |= (1uL << 3) | (1uL << 7);  // Enable GPIOD and GPIOH for MPU
  GPIOH_MODER       &= ~(3uL << (LED0 * 2));     // Clear mode
  GPIOH_MODER       |=  (1uL << (LED0 * 2));     // Set mode to output
  GPIOD_MODER       &= ~(3uL << (LED1 * 2));     // Clear mode
  GPIOD_MODER       |=  (1uL << (LED1 * 2));     // Set mode to output
}

/*********************************************************************
*
*       BSP_SetLED()
*/
void BSP_SetLED(int Index) {
  if (Index == 0) {
    GPIOH_BSRR = (1uL << LED0);
  } else if (Index == 1) {
    GPIOD_BSRR = (1uL << LED1);
  }
}

/*********************************************************************
*
*       BSP_ClrLED()
*/
void BSP_ClrLED(int Index) {
  if (Index == 0) {
    GPIOH_BSRR = (1uL << (LED0 + 16));
  } else if (Index == 1) {
    GPIOD_BSRR = (1uL << (LED1 + 16));
  }
}

/*********************************************************************
*
*       BSP_ToggleLED()
*/
void BSP_ToggleLED(int Index) {
  if (Index == 0) {
    if (GPIOH_ODR & (1uL << LED0)) {
      BSP_ClrLED(Index);
    } else {
      BSP_SetLED(Index);
    }
  } else if (Index == 1) {
    if (GPIOD_ODR & (1uL << LED1)) {
      BSP_ClrLED(Index);
    } else {
      BSP_SetLED(Index);
    }
  }
}

/*************************** End of file ****************************/
