/*****************************************************************************
 *                   SEGGER Microcontroller GmbH & Co. KG                    *
 *            Solutions for real time microcontroller applications           *
 *****************************************************************************
 *                                                                           *
 *               (c) 2015 SEGGER Microcontroller GmbH & Co. KG               *
 *                                                                           *
 *           Internet: www.segger.com   Support: support@segger.com          *
 *                                                                           *
 *****************************************************************************/

function __readMemory32(Addr, MemoryArea) {
  return TargetInterface.peekUint32(Addr);
}

function __writeMemory32(Data, Addr, MemoryArea) {
  TargetInterface.pokeUint32(Addr, Data);
}

function CHGBIT (addr, mask, data) {
  var reg;

  reg  = __readMemory32(addr, "Memory");
  reg &= ~mask;
  reg |=  data;
  __writeMemory32(reg, addr, "Memory");
}

function CLRBIT (addr, mask) {
  var reg;

  reg  = __readMemory32(addr, "Memory");
  reg &= ~mask;
  __writeMemory32(reg, addr, "Memory");
}

function SETBIT (addr, mask) {
  var reg;

  reg  = __readMemory32(addr, "Memory");
  reg |= mask;
  __writeMemory32(reg, addr, "Memory");
}

function TESTBIT (addr, mask) {
  var reg;

  reg = __readMemory32(addr, "Memory");
  return((reg & mask) != 0);
}

/*************************************************************************
 *  int I2C0 module
 *************************************************************************/
function i2c0_init() {
  /*Enable Interface Clock*/
  CHGBIT (0x44E00400, 0x3, 0x2);
  CHGBIT (0x44E0040C, 0x3, 0x2);
  /*Enable Func Clock*/
  CHGBIT (0x44E004B8, 0x3, 0x2);
  /*Wait clocks to get active*/
  while(TESTBIT(0x44E0040C, 0x3<<16) ||
        TESTBIT(0x44E004B8, 0x3<<16));

  /*Reset I2C0 module*/
  SETBIT (0x44E0B010, 0x1<<1);
  while(TESTBIT(0x44E0B010, 0x1<<1));
  /*Disable auto idle*/
  CLRBIT (0x44E0B010, 0x1);
  /*Prescaler set to 2. ICLK is 24MHz*/
  __writeMemory32(2-1, 0x44E0B0B0, "Memory");
  /*Set SCLL */
  __writeMemory32(24, 0x44E0B0B4, "Memory");
  /*Set SCLH */
  __writeMemory32(24, 0x44E0B0B8, "Memory");
  /*Pin Config*/
  /*Enable Control module clocks*/
  CHGBIT (0x44E00404, 0x3, 0x2);
  /*Wait clocks to get active*/
  while(TESTBIT(0x44E00404, 0x3<<16));
  /*I2C0 SDA Pin configure*/
  __writeMemory32(0x70, 0x44E10988, "Memory");
  /*I2C0 SCL Pin configure*/
  __writeMemory32(0x70, 0x44E1098C, "Memory");
  /*I2C0 Enable*/
  SETBIT (0x44E0B0A4, 0x1<<15);
  /*Disable interrupts*/
  SETBIT (0x44E0B030, 0xFFFFFFFF);
}

/*************************************************************************
 * Function Name: i2c0_send
 *************************************************************************/
function i2c0_send(slave,data,cntr,stop) {
  var status;

  /*check for bus busy*/
  while(TESTBIT(0x44E0B024, 0x1<<12));
  /*Clear IRQ Status*/
  SETBIT (0x44E0B028, (0x1<<4) | (0x1<<2) | (0x1<<1));
  /*Enable interrupts*/
  SETBIT (0x44E0B02C, (0x1<<4) | (0x1<<2) | (0x1<<1) | (0x1));
  /*Set Slave Addr*/
  __writeMemory32(slave, 0x44E0B0AC, "Memory");
  /*Master mode*/
  SETBIT (0x44E0B0A4, 0x1<<10);
  /*Transmit mode*/
  SETBIT (0x44E0B0A4, 0x1<<9);
  /*Set data count*/
  __writeMemory32(cntr, 0x44E0B098, "Memory");
  /*Start and stop condition*/
  if(stop) {
    CHGBIT (0x44E0B0A4, 0x3, 0x3);
  } else {
    CHGBIT (0x44E0B0A4, 0x3, 0x1);
  }
  /**/
  do {
    status = __readMemory32(0x44E0B028, "Memory");

    if(status & (0x1)) {
      i2c0_init();
      return -1;
    }
    /*NACK Received*/
    if(status & (0x1<<1)) {
      SETBIT (0x44E0B028, 0x1<<1);
      /*if(!stop)
      {
        CHGBIT (0x44E0B0A4, 0x3, 0x2);
      } */
      return -1;
    }
    if((status & (0x1<<4)) && cntr) {
     /*write data to FIFO*/
      SETBIT (0x44E0B028, 0x1<<4);
      __writeMemory32(data & 0xFF, 0x44E0B09C, "Memory");
      data = data>>8;
      cntr--;
    }
  } while(!TESTBIT(0x44E0B024, 0x1<<2));
  /*Disable interrupts*/
  SETBIT (0x44E0B030, 0xFFFFFFFF);
  return 0;
}
/*************************************************************************
 * Function Name: i2c0_read
  *************************************************************************/
function i2c0_read(slave,cntr) {
  var status, data;

  /*Clear IRQ Status*/
  SETBIT (0x44E0B028, (0x1<<3) | (0x1<<2) | (0x1<<1));
  /*Enable interrupts*/
  SETBIT (0x44E0B02C, (0x1<<3) | (0x1<<2) | (0x1<<1) | (0x1));
  /*Set Slave Addr*/
  __writeMemory32(slave, 0x44E0B0AC, "Memory");
  /*Master mode*/
  SETBIT (0x44E0B0A4, 0x1<<10);
  /*Receive mode*/
  CLRBIT (0x44E0B0A4, 0x1<<9);
  /*Set data count*/
  __writeMemory32(cntr, 0x44E0B098, "Memory");
  /*Start and stop condition*/
  CHGBIT (0x44E0B0A4, 0x3, 0x3);
  /**/
  data = 0;
  do {
    status = __readMemory32(0x44E0B028, "Memory");
    if(status & (0x1)) {
      return -1;
    }
    /*NACK Received*/
    if(status & (0x1<<1)) {
      SETBIT (0x44E0B028, 0x1<<1);
      /*if(!stop)
      {
        CHGBIT (0x44E0B0A4, 0x3, 0x2);
      } */
      return -1;
    }
    if((status & (0x1<<3)) && cntr) {
      /*Read data from FIFO*/
      SETBIT (0x44E0B028, 0x1<<3);
      data = data<<8 & 0xFFFFFF00;
      data |= (__readMemory32(0x44E0B09C, "Memory") & 0xFF);
      cntr--;
    }
   } while(!TESTBIT(0x44E0B024, 0x1<<2));

  /*Disable interrupts*/
  SETBIT (0x44E0B030, 0xFFFFFFFF);
  return data;
}

function init_power() {
  TargetInterface.message("****  Power Init..... \n");
  /*Init I2C0*/
  i2c0_init();
  /*Slecet I2C Control interface*/
  i2c0_send(0x2D, 0x3F, 1, 0);
  /* set SR_CTL_I2C_SEL bit in DEVCTRL_REG*/
  i2c0_send(0x2D, (i2c0_read(0x2D, 1)|(1<<4))<<8 | 0x3F, 2, 1);

  /* VDD1 Set*/
  i2c0_send(0x2D, 0x1521, 2, 1);
  /*VDD1 1.26 V*/
  i2c0_send(0x2D, 0x3822, 2, 1);
}

function get_input_clock_frequency() {
  var temp;
  var clk_in;

  temp = __readMemory32((0x44E10000 + 0x40), "Memory");
  temp = temp >> 22;
  temp = temp & 0x3;
  if(temp == 0) {
    clk_in = 19;  //19.2MHz
    TargetInterface.message("Input Clock Read from SYSBOOT[15:14]:  19.2MHz\n");
  }
  if(temp == 1) {
    clk_in = 24;  //24MHz
    TargetInterface.message("Input Clock Read from SYSBOOT[15:14]:  24MHz\n");
  }
  if(temp == 2) {
    clk_in = 25;  //25MHz
    TargetInterface.message("Input Clock Read from SYSBOOT[15:14]:  25MHz\n");
  }
  if(temp == 3) {
    clk_in = 26;  //26MHz
    TargetInterface.message("Input Clock Read from SYSBOOT[15:14]:  26MHz\n");
  }
  return clk_in;
}

function mpu_pll_config( clk_in,  N,  M,  M2) {
  var ref_clk,clk_out;
  var clkmode,clksel,div_m2,idlest_dpll;
  var temp,i;

  ref_clk     = clk_in/(N+1);
  clk_out     = (ref_clk*M)/M2;
  clkmode=__readMemory32((0x44E00000 + 0x488), "Memory");
  clksel= __readMemory32((0x44E00000 + 0x42C), "Memory");
  div_m2= __readMemory32((0x44E00000 + 0x4A8), "Memory");
  TargetInterface.message("****  Going to Bypass... \n");
  //put the DPLL in bypass mode
  __writeMemory32(0x4, (0x44E00000 + 0x488), "Memory");
  while(((__readMemory32((0x44E00000 + 0x420), "Memory") & 0x101) != 0x00000100)); //wait for bypass status
  TargetInterface.message("****  Bypassed, changing values... \n");
  //set multiply and divide values
  clksel = clksel & (~0x7FFFF);
  clksel = clksel | ((M <<0x8) | N);
  __writeMemory32(clksel, (0x44E00000 + 0x42C), "Memory");
  div_m2 = div_m2 & ~0x1F;
  div_m2 = div_m2 | M2;
  __writeMemory32(div_m2, (0x44E00000 + 0x4A8), "Memory");
  TargetInterface.message("****  Locking ARM PLL\n");
  //now lock the DPLL
  clkmode = clkmode | 0x7;  //enables lock mode
  __writeMemory32(clkmode, (0x44E00000 + 0x488), "Memory");
  while(((__readMemory32((0x44E00000 + 0x420), "Memory") & 0x101) != 0x1)); //wait for lock
}


function core_pll_config( clk_in,  N,  M,  M4,  M5,  M6) {
  var ref_clk,clk_out4,clk_out5,clk_out6;
  var clkmode,clksel,div_m4,div_m5,div_m6,idlest_dpll;

  ref_clk     = clk_in/(N+1);
  clk_out4     = (ref_clk*M)/M4;  //M4=200MHz
  clk_out5     = (ref_clk*M)/M5;  //M5=250MHz
  clk_out6     = (ref_clk*M)/M6;  //M6=500MHz
  clkmode= __readMemory32((0x44E00000 + 0x490), "Memory");
  clksel= __readMemory32((0x44E00000 + 0x468), "Memory");
  div_m4= __readMemory32((0x44E00000 + 0x480), "Memory");
  div_m5= __readMemory32((0x44E00000 + 0x484), "Memory");
  div_m6= __readMemory32((0x44E00000 + 0x4D8), "Memory");
  //put DPLL in bypass mode
  clkmode = (clkmode & 0xfffffff8)|0x00000004;
  __writeMemory32(clkmode, (0x44E00000 + 0x490), "Memory");
  while((__readMemory32((0x44E00000 + 0x45C),  "Memory") & 0x00000100 )!=0x00000100); //wait for bypass status
  TargetInterface.message("****  Core Bypassed\n");
  //set multiply and divide values
  clksel = clksel & (~0x7FFFF);
  clksel = clksel | ((M <<0x8) | N);
  __writeMemory32(clksel, (0x44E00000 + 0x468), "Memory");
  div_m4= M4;   //200MHz
  __writeMemory32(div_m4, (0x44E00000 + 0x480), "Memory");
  div_m5=  M5;  //250MHz
  __writeMemory32(div_m5, (0x44E00000 + 0x484), "Memory");
  div_m6=  M6;  //500MHz
  __writeMemory32(div_m6, (0x44E00000 + 0x4D8), "Memory");
  TargetInterface.message("****  Now locking Core...\n");
  //now lock the PLL
  clkmode =(clkmode&0xfffffff8)|0x00000007;
  __writeMemory32(clkmode, (0x44E00000 + 0x490), "Memory");
  while((__readMemory32((0x44E00000 + 0x45C), "Memory") & 0x00000001 )!=0x00000001);
  TargetInterface.message("****  Core locked\n");
}

function ddr_pll_config( clk_in,  N,  M,  M2) {
  var ref_clk,clk_out ;
  var clkmode,clksel,div_m2,idlest_dpll;

  ref_clk     = clk_in/(N+1);
  clk_out     = (ref_clk*M)/M2;
  clkmode=__readMemory32((0x44E00000 + 0x494), "Memory");
  clksel= __readMemory32((0x44E00000 + 0x440), "Memory");
  div_m2= __readMemory32((0x44E00000 + 0x4A0), "Memory");
  clkmode =(clkmode&0xfffffff8)|0x00000004;
  __writeMemory32(clkmode, (0x44E00000 + 0x494), "Memory");
  while((__readMemory32((0x44E00000 + 0x434), "Memory") & 0x00000100 )!=0x00000100);
  TargetInterface.message("****  DDR DPLL Bypassed\n");
  clksel = clksel & (~0x7FFFF);
  clksel = clksel | ((M <<0x8) | N);
  __writeMemory32(clksel, (0x44E00000 + 0x440), "Memory");
  div_m2 = __readMemory32((0x44E00000 + 0x4A0), "Memory");
  div_m2 = (div_m2&0xFFFFFFE0) | M2;
  __writeMemory32(div_m2, (0x44E00000 + 0x4A0), "Memory");
  clkmode =(clkmode&0xfffffff8)|0x00000007;
  __writeMemory32(clkmode, (0x44E00000 + 0x494), "Memory");
  while((__readMemory32((0x44E00000 + 0x434), "Memory") & 0x00000001 )!=0x00000001);
  TargetInterface.message("****  DDR DPLL Locked\n");
}

function per_pll_config( clk_in,  N,  M,  M2) {
  var ref_clk,clk_out;
  var clkmode,clksel,div_m2,idlest_dpll;

  ref_clk     = clk_in/(N+1);
  clk_out     = (ref_clk*M)/M2;
  clkmode=__readMemory32((0x44E00000 + 0x48C), "Memory");
  clksel= __readMemory32((0x44E00000 + 0x49C), "Memory");
  div_m2= __readMemory32((0x44E00000 + 0x4AC), "Memory");
  clkmode =(clkmode&0xfffffff8)|0x00000004;
  __writeMemory32(clkmode, (0x44E00000 + 0x48C), "Memory");
  while((__readMemory32((0x44E00000 + 0x470), "Memory") & 0x00000100 )!=0x00000100);
  TargetInterface.message("****  PER DPLL Bypassed\n");
  clksel = clksel & (~0x7FFFF);
  clksel = clksel | ((M <<0x8) | N);
  __writeMemory32(clksel, (0x44E00000 + 0x49C), "Memory");
  div_m2= 0xFFFFFF80 | M2;
  __writeMemory32(div_m2, (0x44E00000 + 0x4AC), "Memory");
  clkmode =(clkmode&0xfffffff8)|0x00000007;
  __writeMemory32(clkmode,(0x44E00000 + 0x48C), "Memory");
  while((__readMemory32((0x44E00000 + 0x470), "Memory") & 0x00000001 )!=0x00000001);
  TargetInterface.message("****  PER DPLL Locked\n");
}

function disp_pll_config( clk_in,  N,  M,  M2) {
  var ref_clk,clk_out;
  var clkmode,clksel,div_m2,idlest_dpll;

  TargetInterface.message("****  DISP PLL Config is in progress .......... \n");
  ref_clk     = clk_in/(N+1);
  clk_out     = (ref_clk*M)/M2;
  clkmode=__readMemory32((0x44E00000 + 0x498), "Memory");
  clksel= __readMemory32((0x44E00000 + 0x454), "Memory");
  div_m2= __readMemory32((0x44E00000 + 0x4A4), "Memory");
  clkmode =(clkmode&0xfffffff8)|0x00000004;
  __writeMemory32(clkmode, (0x44E00000 + 0x498), "Memory");
  while((__readMemory32((0x44E00000 + 0x448), "Memory") & 0x00000100 )!=0x00000100);
  clksel = clksel & (~0x7FFFF);
  clksel = clksel | ((M <<0x8) | N);
  __writeMemory32(clksel, (0x44E00000 + 0x454), "Memory");
  div_m2= 0xFFFFFFE0 | M2;
  __writeMemory32(div_m2, (0x44E00000 + 0x4A4), "Memory");
  clkmode =(clkmode&0xfffffff8)|0x00000007;
  __writeMemory32(clkmode, (0x44E00000 + 0x498), "Memory");
  while((__readMemory32((0x44E00000 + 0x448), "Memory") & 0x00000001 )!=0x00000001);
  TargetInterface.message("****  DISP PLL Config is DONE .......... \n");
}

function arm_opp120_config() {
  var clk_in;
  TargetInterface.message("****  Subarctic ALL ADPLL Config for OPP == OPP100 is In Progress ......... \n");
  init_power();

  clk_in = get_input_clock_frequency();
  if (clk_in == 24) {
    mpu_pll_config(clk_in, 23, 600, 1);
    core_pll_config(clk_in, 23, 1000, 10, 8, 4);
    ddr_pll_config(clk_in, 23, 266, 1);
    per_pll_config(clk_in, 23, 960, 5);
    disp_pll_config(clk_in, 23, 48, 1);
    TargetInterface.message("****  Subarctic ALL ADPLL Config for OPP == OPP100 is Done ......... \n");
  } else {
    TargetInterface.message("****  Subarctic PLL Config failed!!  Check SYSBOOT[15:14] for proper input freq config \n");
  }
}

function emif_prcm_clk_enable() {
   TargetInterface.message("EMIF PRCM is in progress ....... \n");
  __writeMemory32(0x2, (0x44E00000 + 0x0D0), "Memory");
  __writeMemory32(0x2, (0x44E00000 + 0x028), "Memory");
  while(__readMemory32((0x44E00000 + 0x028), "Memory")!= 0x02);
  TargetInterface.message("EMIF PRCM Done \n");
}

function vtt_enable() {
  /*VTT Enable*/
  /*Enable Control module clocks*/
  CHGBIT(0x44E00400, 0x3, 0x2);
  CHGBIT(0x44E00404, 0x3, 0x2);
  /*Wait clock to get active*/
  while(TESTBIT(0x44E00404, 0x3<<16));
  /*CONF_ECAP0_IN_PWM0_OUT (GPIO0_7) Pin configure*/
  /*pull disabled*/
  /*Select GPIO mode*/
  __writeMemory32(0x0000002F,0x44E10964, "Memory");
  /*Enable GPIO Interface Clock*/
  CHGBIT(0x44E0040C, 0x3, 0x2);
  CHGBIT(0x44E00408, 0x3, 0x2);
  /*Wait clocks to get active*/
  while(TESTBIT(0x44E0040C, 0x3<<16) ||
        TESTBIT(0x44E00408, 0x3<<16));
  /*Make VTT_EN (GPIO0_7) pin output*/
  CLRBIT(0x44E07134,(1<<7));
  /*Set VTT_EN*/
  SETBIT(0x44E07194,(1<<7));
}


function vtp_enable() {
  __writeMemory32((__readMemory32(((0x44E10000) + 0x0E0C), "Memory") | 0x00000040),((0x44E10000) + 0x0E0C) , "Memory");
  __writeMemory32((__readMemory32(((0x44E10000) + 0x0E0C), "Memory") & 0xFFFFFFFE),((0x44E10000) + 0x0E0C) ,"Memory");
  __writeMemory32((__readMemory32(((0x44E10000) + 0x0E0C), "Memory") | 0x00000001),((0x44E10000) + 0x0E0C) , "Memory");
  TargetInterface.message("Waiting for VTP Ready .......\n");
  while((__readMemory32(((0x44E10000) + 0x0E0C), "Memory") & 0x00000020) != 0x00000020);
}

function cmd_macro_config( REG_PHY_CTRL_SLAVE_RATIO_value, CMD_REG_PHY_CTRL_SLAVE_FORCE_value, CMD_REG_PHY_CTRL_SLAVE_DELAY_value, PHY_DLL_LOCK_DIFF_value, CMD_PHY_INVERT_CLKOUT_value) {
  TargetInterface.message("\DDR PHY CMD0 Register configuration is in progress ....... \n");
  __writeMemory32(REG_PHY_CTRL_SLAVE_RATIO_value, (0x01C + (0x44E12000)), "Memory");
  __writeMemory32(CMD_REG_PHY_CTRL_SLAVE_FORCE_value, (0x020 + (0x44E12000)), "Memory");
  __writeMemory32(CMD_REG_PHY_CTRL_SLAVE_DELAY_value, (0x024 + (0x44E12000)), "Memory");
  __writeMemory32(PHY_DLL_LOCK_DIFF_value, (0x028 + (0x44E12000)), "Memory");
  __writeMemory32(CMD_PHY_INVERT_CLKOUT_value, (0x02C + (0x44E12000)), "Memory");
  TargetInterface.message("\DDR PHY CMD1 Register configuration is in progress ....... \n");
  __writeMemory32(REG_PHY_CTRL_SLAVE_RATIO_value, (0x050 + (0x44E12000)), "Memory");
  __writeMemory32(CMD_REG_PHY_CTRL_SLAVE_FORCE_value, (0x054 + (0x44E12000)), "Memory");
  __writeMemory32(CMD_REG_PHY_CTRL_SLAVE_DELAY_value, (0x058 + (0x44E12000)), "Memory");
  __writeMemory32(PHY_DLL_LOCK_DIFF_value, (0x05C + (0x44E12000)), "Memory");
  __writeMemory32(CMD_PHY_INVERT_CLKOUT_value, (0x060 + (0x44E12000)), "Memory");
  TargetInterface.message("\DDR PHY CMD2 Register configuration is in progress ....... \n");
  __writeMemory32(REG_PHY_CTRL_SLAVE_RATIO_value, (0x084 + (0x44E12000)), "Memory");
  __writeMemory32(CMD_REG_PHY_CTRL_SLAVE_FORCE_value, (0x088 + (0x44E12000)), "Memory");
  __writeMemory32(CMD_REG_PHY_CTRL_SLAVE_DELAY_value, (0x08C + (0x44E12000)), "Memory");
  __writeMemory32(PHY_DLL_LOCK_DIFF_value, (0x090 + (0x44E12000)), "Memory");
  __writeMemory32(CMD_PHY_INVERT_CLKOUT_value, (0x094 + (0x44E12000)), "Memory");
}

function data_macro_config( dataMacroNum, PHY_RD_DQS_SLAVE_RATIO_value, PHY_WR_DQS_SLAVE_RATIO_value, REG_PHY_WRLVL_INIT_RATIO_value,
       REG_PHY_GATELVL_INIT_RATIO_value, REG_PHY_FIFO_WE_SLAVE_RATIO_value, REG_PHY_WR_DATA_SLAVE_RATIO_value) {
var offset;
  if(dataMacroNum == 0) {
    offset = 0x00;
    TargetInterface.message("DDR PHY DATA0 Register configuration is in progress ....... \n");
  } else if(dataMacroNum == 1) {
    offset = 0xA4;
    TargetInterface.message("DDR PHY DATA1 Register configuration is in progress ....... \n");
  }

  __writeMemory32(((PHY_RD_DQS_SLAVE_RATIO_value<<30)|(PHY_RD_DQS_SLAVE_RATIO_value<<20)|(PHY_RD_DQS_SLAVE_RATIO_value<<10)|(PHY_RD_DQS_SLAVE_RATIO_value)), ((0x0C8 + (0x44E12000)) + offset), "Memory");
  __writeMemory32(PHY_RD_DQS_SLAVE_RATIO_value>>2, ((0x0CC + (0x44E12000)) + offset), "Memory");

    __writeMemory32(((PHY_WR_DQS_SLAVE_RATIO_value<<30)|(PHY_WR_DQS_SLAVE_RATIO_value<<20)|(PHY_WR_DQS_SLAVE_RATIO_value<<10)|(PHY_WR_DQS_SLAVE_RATIO_value)), ((0x0DC + (0x44E12000)) + offset), "Memory");
  __writeMemory32(PHY_WR_DQS_SLAVE_RATIO_value>>2, ((0x0E0 + (0x44E12000)) + offset), "Memory");

  __writeMemory32(((REG_PHY_WRLVL_INIT_RATIO_value<<30)|(REG_PHY_WRLVL_INIT_RATIO_value<<20)|(REG_PHY_WRLVL_INIT_RATIO_value<<10)|(REG_PHY_WRLVL_INIT_RATIO_value)), ((0x0F0 + (0x44E12000)) + offset), "Memory");
  __writeMemory32(REG_PHY_WRLVL_INIT_RATIO_value>>2, ((0x0F4 + (0x44E12000)) + offset), "Memory");

    __writeMemory32(((REG_PHY_GATELVL_INIT_RATIO_value<<30)|(REG_PHY_GATELVL_INIT_RATIO_value<<20)|(REG_PHY_GATELVL_INIT_RATIO_value<<10)|(REG_PHY_GATELVL_INIT_RATIO_value)), ((0x0FC + (0x44E12000)) + offset), "Memory");
  __writeMemory32(REG_PHY_GATELVL_INIT_RATIO_value>>2, ((0x100 + (0x44E12000)) + offset), "Memory");

  __writeMemory32(((REG_PHY_FIFO_WE_SLAVE_RATIO_value<<30)|(REG_PHY_FIFO_WE_SLAVE_RATIO_value<<20)|(REG_PHY_FIFO_WE_SLAVE_RATIO_value<<10)|(REG_PHY_FIFO_WE_SLAVE_RATIO_value)), ((0x108 + (0x44E12000)) + offset), "Memory");
  __writeMemory32(REG_PHY_FIFO_WE_SLAVE_RATIO_value>>2,((0x10C + (0x44E12000)) + offset), "Memory");

  __writeMemory32(((REG_PHY_WR_DATA_SLAVE_RATIO_value<<30)|(REG_PHY_WR_DATA_SLAVE_RATIO_value<<20)|(REG_PHY_WR_DATA_SLAVE_RATIO_value<<10)|(REG_PHY_WR_DATA_SLAVE_RATIO_value)),((0x120 + (0x44E12000)) + offset), "Memory");
  __writeMemory32(REG_PHY_WR_DATA_SLAVE_RATIO_value>>2, ((0x124 + (0x44E12000)) + offset), "Memory");
  __writeMemory32(0x0,((0x138 + (0x44E12000)) + offset), "Memory");
}

function emif_mmr_config( Read_Latency, Timing1, Timing2, Timing3, Sdram_Config, Ref_Ctrl, Zq_Config) {
  var i;

  TargetInterface.message("emif Timing register configuration is in progress ....... \n");
  __writeMemory32(Read_Latency, (0x4C000000 + 0x0E4), "Memory");
  __writeMemory32(Read_Latency, (0x4C000000 + 0x0E8), "Memory");
  __writeMemory32(Read_Latency, (0x4C000000 + 0x0EC), "Memory");
  __writeMemory32(Timing1, (0x4C000000 + 0x018), "Memory");
  __writeMemory32(Timing1, (0x4C000000 + 0x01C), "Memory");
  __writeMemory32(Timing2, (0x4C000000 + 0x020), "Memory");
  __writeMemory32(Timing2, (0x4C000000 + 0x024), "Memory");
  __writeMemory32(Timing3, (0x4C000000 + 0x028), "Memory");
  __writeMemory32(Timing3, (0x4C000000 + 0x02C), "Memory");
  __writeMemory32(Zq_Config, (0x4C000000 + 0x0C8), "Memory");
  __writeMemory32(Ref_Ctrl, (0x4C000000 + 0x010), "Memory");
  __writeMemory32(Ref_Ctrl, (0x4C000000 + 0x014), "Memory");
  __writeMemory32(Sdram_Config, (0x4C000000 + 0x008), "Memory");
  __writeMemory32(Sdram_Config, (0x4C000000 + 0x00C), "Memory");
  TargetInterface.message("emif Timing register configuration is done ....... \n");
}

function  cmd_DDR3_EMIF_Config( REG_PHY_CTRL_SLAVE_RATIO, CMD_REG_PHY_CTRL_SLAVE_FORCE, CMD_REG_PHY_CTRL_SLAVE_DELAY, CMD_PHY_DLL_LOCK_DIFF, CMD_PHY_INVERT_CLKOUT, DATA_PHY_RD_DQS_SLAVE_RATIO, DATA_PHY_WR_DQS_SLAVE_RATIO, DATA_REG_PHY_WRLVL_INIT_RATIO, DATA_REG_PHY_GATELVL_INIT_RATIO, DATA_REG_PHY_FIFO_WE_SLAVE_RATIO, DATA_REG_PHY_WR_DATA_SLAVE_RATIO, RD_LATENCY, TIMING1, TIMING2, TIMING3, SDRAM_CONFIG, REF_CTRL, ZQ_CONFIG) {
  emif_prcm_clk_enable();
  TargetInterface.message("DDR PHY Configuration In progress \n");
  vtt_enable();
  vtp_enable();
  cmd_macro_config(REG_PHY_CTRL_SLAVE_RATIO,CMD_REG_PHY_CTRL_SLAVE_FORCE,CMD_REG_PHY_CTRL_SLAVE_DELAY,CMD_PHY_DLL_LOCK_DIFF,CMD_PHY_INVERT_CLKOUT);
  data_macro_config(0,DATA_PHY_RD_DQS_SLAVE_RATIO,DATA_PHY_WR_DQS_SLAVE_RATIO,DATA_REG_PHY_WRLVL_INIT_RATIO, DATA_REG_PHY_GATELVL_INIT_RATIO,DATA_REG_PHY_FIFO_WE_SLAVE_RATIO,DATA_REG_PHY_WR_DATA_SLAVE_RATIO);
  data_macro_config(1,DATA_PHY_RD_DQS_SLAVE_RATIO,DATA_PHY_WR_DQS_SLAVE_RATIO,DATA_REG_PHY_WRLVL_INIT_RATIO, DATA_REG_PHY_GATELVL_INIT_RATIO,DATA_REG_PHY_FIFO_WE_SLAVE_RATIO,DATA_REG_PHY_WR_DATA_SLAVE_RATIO);
  __writeMemory32(0x00, (0x134 + (0x44E12000)), "Memory");
  __writeMemory32(0x00, (0x1D8 + (0x44E12000)), "Memory");
  __writeMemory32((0x18B), ((0x44E10000) + 0x1404), "Memory");
  __writeMemory32((0x18B), ((0x44E10000) + 0x1408), "Memory");
  __writeMemory32((0x18B), ((0x44E10000) + 0x140C), "Memory");
  __writeMemory32((0x18B), ((0x44E10000) + 0x1440), "Memory");
  __writeMemory32((0x18B), ((0x44E10000) + 0x1444), "Memory");
  __writeMemory32(__readMemory32(((0x44E10000) + 0x0E04), "Memory") & ~0x10000000, ((0x44E10000) + 0x0E04), "Memory");
  __writeMemory32(__readMemory32(((0x44E10000) + 0x131C), "Memory") | 0x00000001, ((0x44E10000) + 0x131C), "Memory");
  emif_mmr_config(RD_LATENCY,TIMING1,TIMING2,TIMING3,SDRAM_CONFIG,REF_CTRL, ZQ_CONFIG);
  if((__readMemory32((0x4C000000 + 0x004), "Memory") & 0x4) == 0x4) {
    TargetInterface.message("PHY is READY!!\n");
  }
  TargetInterface.message("DDR PHY Configuration done \n");
}

function ddr3_emif_config() {
  TargetInterface.message("****  AM335x OPP120 DDR2 EMIF and PHY configuration is in progress......... \n");


  cmd_DDR3_EMIF_Config(0x40,  0x00, 0x00, 0x4,
                        0x1,  0x3B, 0x85,   0,
                         0,  0x100, 0xC1,0x06,
                 0x0888A39B, 0x26337FDA, 0x501F830F, 0x61C04AB2,
                 0x0000093B, 0x50074BE4);
  TargetInterface.message("****  AM335x OPP120 DDR2 EMIF and PHY configuration is done......... \n");
}

function am335x_evm_initialization() {
  var psc_base;
  var reg;
  var module_offest;

  TargetInterface.message(" AM335x 15x15 EVM Initialization is in progress .......... \n");
  arm_opp120_config();
  ddr3_emif_config();
  TargetInterface.message(" AM335x 15x15 EVM Initialization is done .......... \n");
}


function Reset() {
  TargetInterface.resetAndStop(200);
  am335x_evm_initialization();
  TargetInterface.stop(200);
}

